@@ -1,23 +1,23 @@
-// -*- c++ -*- (enables emacs c++ mode)
-//===========================================================================
-//
-// Copyright (C) 2006-2008 Yves Renard, Julien Pommier.
-//
-// This file is a part of GETFEM++
-//
-// Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
-// under  the  terms  of the  GNU  Lesser General Public License as published
-// by  the  Free Software Foundation;  either version 2.1 of the License,  or
-// (at your option) any later version.
-// This program  is  distributed  in  the  hope  that it will be useful,  but
-// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-// or  FITNESS  FOR  A PARTICULAR PURPOSE.  See the GNU Lesser General Public
-// License for more details.
-// You  should  have received a copy of the GNU Lesser General Public License
-// along  with  this program;  if not, write to the Free Software Foundation,
-// Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
-//
-//===========================================================================
+/*===========================================================================
+ 
+ Copyright (C) 2006-2012 Yves Renard, Julien Pommier.
+ 
+ This file is a part of GETFEM++
+ 
+ Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+ under  the  terms  of the  GNU  Lesser General Public License as published
+ by  the  Free Software Foundation;  either version 3 of the License,  or
+ (at your option) any later version along with the GCC Runtime Library
+ Exception either version 3.1 or (at your option) any later version.
+ This program  is  distributed  in  the  hope  that it will be useful,  but
+ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ or  FITNESS  FOR  A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+ License and GCC Runtime Library Exception for more details.
+ You  should  have received a copy of the GNU Lesser General Public License
+ along  with  this program;  if not, write to the Free Software Foundation,
+ Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
+ 
+===========================================================================*/
 
 #ifdef MAINTAINER_MODE
 # include <sys/types.h>
@@ -693,20 +693,33 @@ namespace getfemint {
 
   
   const getfem::abstract_hyperelastic_law &
-  abstract_hyperelastic_law_from_name(const std::string &lawname) {
-
+  abstract_hyperelastic_law_from_name(const std::string &lawname,
+				      size_type N) {
     static getfem::SaintVenant_Kirchhoff_hyperelastic_law SVK_AHL;
     static getfem::Mooney_Rivlin_hyperelastic_law MR_AHL;
     static getfem::Ciarlet_Geymonat_hyperelastic_law CG_AHL;
-
+    static getfem::generalized_Blatz_Ko_hyperelastic_law GBK_AHL;
+    static getfem::plane_strain_hyperelastic_law PS_SVK_AHL(&SVK_AHL);
+    static getfem::plane_strain_hyperelastic_law PS_MR_AHL(&MR_AHL);
+    static getfem::plane_strain_hyperelastic_law PS_CG_AHL(&CG_AHL);
+    static getfem::plane_strain_hyperelastic_law PS_GBK_AHL(&GBK_AHL);
+    
     if (cmd_strmatch(lawname, "SaintVenant Kirchhoff") ||
-	cmd_strmatch(lawname, "svk")) return SVK_AHL;
+	cmd_strmatch(lawname, "svk"))
+      { if (N == 2) return PS_SVK_AHL; else return SVK_AHL; }
 
     if (cmd_strmatch(lawname, "Mooney Rivlin") ||
-	cmd_strmatch(lawname, "mr")) return MR_AHL;
+	cmd_strmatch(lawname, "mr"))
+      { if (N == 2) return PS_MR_AHL; else return MR_AHL; }
 
     if (cmd_strmatch(lawname, "Ciarlet Geymonat") ||
-	cmd_strmatch(lawname, "cg")) return CG_AHL;
+	cmd_strmatch(lawname, "cg"))
+      { if (N == 2) return PS_CG_AHL; else return CG_AHL; }
+
+    if (cmd_strmatch(lawname, "generalized Blatz Ko") ||
+	cmd_strmatch(lawname, "gbk"))
+      {	if (N == 2) return PS_GBK_AHL; else return GBK_AHL; }
+    
 
     THROW_BADARG(lawname <<
 		 " is not the name of a known hyperelastic law. \\"
