@@ -1,23 +1,23 @@
-// -*- c++ -*- (enables emacs c++ mode)
-//===========================================================================
-//
-// Copyright (C) 1999-2008 Yves Renard
-//
-// This file is a part of GETFEM++
-//
-// Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
-// under  the  terms  of the  GNU  Lesser General Public License as published
-// by  the  Free Software Foundation;  either version 2.1 of the License,  or
-// (at your option) any later version.
-// This program  is  distributed  in  the  hope  that it will be useful,  but
-// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-// or  FITNESS  FOR  A PARTICULAR PURPOSE.  See the GNU Lesser General Public
-// License for more details.
-// You  should  have received a copy of the GNU Lesser General Public License
-// along  with  this program;  if not, write to the Free Software Foundation,
-// Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
-//
-//===========================================================================
+/*===========================================================================
+ 
+ Copyright (C) 1999-2012 Yves Renard
+ 
+ This file is a part of GETFEM++
+ 
+ Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+ under  the  terms  of the  GNU  Lesser General Public License as published
+ by  the  Free Software Foundation;  either version 3 of the License,  or
+ (at your option) any later version along with the GCC Runtime Library
+ Exception either version 3.1 or (at your option) any later version.
+ This program  is  distributed  in  the  hope  that it will be useful,  but
+ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ or  FITNESS  FOR  A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+ License and GCC Runtime Library Exception for more details.
+ You  should  have received a copy of the GNU Lesser General Public License
+ along  with  this program;  if not, write to the Free Software Foundation,
+ Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
+ 
+===========================================================================*/
 
 #include "getfem/dal_singleton.h"
 #include "gmm/gmm_condition_number.h"
@@ -83,7 +83,7 @@ namespace getfem {
 		giv.invert(pt, barycentre);
 
 	
-		if (pgt->convex_ref()->is_in_face(f, barycentre) < 0.1)
+		if (gmm::abs(pgt->convex_ref()->is_in_face(f, barycentre)) < 0.1)
 		  r.add(icv[jc], fsub);
 	      }
 	    }
@@ -108,7 +108,7 @@ namespace getfem {
 	      giv.invert(pt, barycentre);
 	
 	      for (short_type f = 0; f < pgt->structure()->nb_faces(); ++f)
-		if (pgt->convex_ref()->is_in_face(f, barycentre) < 0.1)
+		if (gmm::abs(pgt->convex_ref()->is_in_face(f, barycentre)) < 0.1)
 		  { r.add(ic, f); break; }
 	  }
 	}
@@ -138,6 +138,7 @@ namespace getfem {
       else {
 	int ne = int(nb_convex());
 	std::vector<int> xadj(ne+1), adjncy, numelt(ne), npart(ne);
+	std::vector<int> indelt(convex_index().last_true()+1);
 	
 	double t_ref = MPI_Wtime();
 
@@ -145,10 +146,14 @@ namespace getfem {
 	ind_set s;
 	for (dal::bv_visitor ic(convex_index()); !ic.finished(); ++ic, ++j) {
 	  numelt[j] = ic;
+	  indelt[ic] = j;
+	}
+	j = 0;
+	for (dal::bv_visitor ic(convex_index()); !ic.finished(); ++ic, ++j) {
 	  xadj[j] = k;
 	  neighbours_of_convex(ic, s);
 	  for (ind_set::iterator it = s.begin();
-	       it != s.end(); ++it) { adjncy.push_back(*it); ++k; }
+	       it != s.end(); ++it) { adjncy.push_back(indelt[*it]); ++k; }
 	}
 	xadj[j] = k;
 
@@ -363,10 +368,24 @@ namespace getfem {
     }
     return r;
   }
+  
+  scalar_type mesh::maximal_convex_radius_estimate() const {
+    if (convex_index().empty()) return 1;
+    scalar_type r = convex_radius_estimate(convex_index().first_true());
+    for (dal::bv_visitor cv(convex_index()); !cv.finished(); ++cv) {
+      r = std::max(r, convex_radius_estimate(cv));
+    }
+    return r;
+  }
 
   void mesh::copy_from(const mesh& m) {
     clear();
     bgeot::basic_mesh::operator=(m);
+    cvf_sets = m.cvf_sets;
+    valid_cvf_sets = m.valid_cvf_sets;
+    for (std::map<size_type, mesh_region>::iterator it = cvf_sets.begin();
+	 it != cvf_sets.end(); ++it)
+      if (it->second.get_parent_mesh() != 0) it->second.set_parent_mesh(this);
     cvs_v_num.clear();
     gmm::uint64_type d = act_counter();
     for (dal::bv_visitor i(convex_index()); !i.finished(); ++i)
@@ -583,7 +602,7 @@ namespace getfem {
     return bgeot::mesh_structure::memsize() - sizeof(bgeot::mesh_structure)
       + pts.memsize() + (pts.index().last_true()+1)*dim()*sizeof(scalar_type)
       + sizeof(mesh) + trans_exists.memsize() + gtab.memsize()
-      + cvf_sets.memsize() + valid_cvf_sets.memsize();
+      + valid_cvf_sets.card()*sizeof(mesh_region) + valid_cvf_sets.memsize();
   }
 
   struct equilateral_to_GT_PK_grad_aux : public std::vector<base_matrix> {};
