@@ -1,23 +1,23 @@
-// -*- c++ -*- (enables emacs c++ mode)
-//===========================================================================
-//
-// Copyright (C) 2002-2008 Yves Renard, Julien Pommier, Houari Khenous.
-//
-// This file is a part of GETFEM++
-//
-// Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
-// under  the  terms  of the  GNU  Lesser General Public License as published
-// by  the  Free Software Foundation;  either version 2.1 of the License,  or
-// (at your option) any later version.
-// This program  is  distributed  in  the  hope  that it will be useful,  but
-// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-// or  FITNESS  FOR  A PARTICULAR PURPOSE.  See the GNU Lesser General Public
-// License for more details.
-// You  should  have received a copy of the GNU Lesser General Public License
-// along  with  this program;  if not, write to the Free Software Foundation,
-// Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
-//
-//===========================================================================
+/*===========================================================================
+ 
+ Copyright (C) 2002-2012 Yves Renard, Julien Pommier, Houari Khenous.
+ 
+ This file is a part of GETFEM++
+ 
+ Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+ under  the  terms  of the  GNU  Lesser General Public License as published
+ by  the  Free Software Foundation;  either version 3 of the License,  or
+ (at your option) any later version along with the GCC Runtime Library
+ Exception either version 3.1 or (at your option) any later version.
+ This program  is  distributed  in  the  hope  that it will be useful,  but
+ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ or  FITNESS  FOR  A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+ License and GCC Runtime Library Exception for more details.
+ You  should  have received a copy of the GNU Lesser General Public License
+ along  with  this program;  if not, write to the Free Software Foundation,
+ Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
+ 
+===========================================================================*/
 
 /**
  * Dynamic friction in linear elasticity.
@@ -37,6 +37,9 @@
 #include "gmm/gmm.h"
 #include <fstream>
 
+using std::endl; using std::cout; using std::cerr;
+using std::ends; using std::cin;
+
 /* some Getfem++ types that we will be using */
 using bgeot::base_small_vector; /* special class for small (dim<16) vectors */
 using bgeot::base_node;  /* geometrical nodes(derived from base_small_vector)*/
@@ -73,15 +76,15 @@ struct friction_problem {
 
   scalar_type residual;      /* max residual for the iterative solvers       */
   int scheme;  /* 0 = theta method, 1 = Newmark, 2 = middle point,           */
-               /* 3 = middle point with separated contact forces.            */
+               /* 3 = middle point with implicited contact forces.           */
   size_type N, noisy, nocontact_mass;
   scalar_type beta, theta, gamma, restit;
   scalar_type T, dt, r;
   scalar_type init_vert_pos, init_vert_speed, hspeed, dtexport;
   scalar_type pert_stationary, Dirichlet_ratio;
-  bool dt_adapt, periodic, dxexport, Dirichlet, init_stationary;
+  bool dt_adapt, periodic, dxexport, Dirichlet, init_stationary, compare;
 
-  std::string datafilename;
+  std::string datafilename, refrootname, INTEGRATION;
   bgeot::md_param PARAM;
 
   void stationary(plain_vector &U0, plain_vector &LN, plain_vector &LT);
@@ -95,7 +98,7 @@ struct friction_problem {
  */
 void friction_problem::init(void) {
   std::string FEM_TYPE  = PARAM.string_value("FEM_TYPE","FEM name");
-  std::string INTEGRATION = PARAM.string_value("INTEGRATION",
+  INTEGRATION = PARAM.string_value("INTEGRATION",
 					       "Name of integration method");
   cout << "FEM_TYPE="  << FEM_TYPE << "\n";
   cout << "INTEGRATION=" << INTEGRATION << "\n";
@@ -108,6 +111,9 @@ void friction_problem::init(void) {
   mesh.optimize_structure();
 
   datafilename = PARAM.string_value("ROOTFILENAME","Base name of data files.");
+  refrootname = PARAM.string_value("REFROOTNAME",
+				   "Base name of reference files.");
+  compare = (PARAM.int_value("COMPARE") != 0);
   residual = PARAM.real_value("RESIDUAL");
   if (residual == 0.) residual = 1e-10;
 
@@ -281,7 +287,7 @@ void friction_problem::stationary(plain_vector &U0, plain_vector &LN,
   FRICTION.set_stationary(true);
 
   cout << "Computation of the stationary problem\n";
-  gmm::iteration iter(residual, int(noisy), 40000);
+  gmm::iteration iter(residual, int(noisy), 10000);
   getfem::standard_solve(MS, PERIODIC, iter);
 
   gmm::copy(ELAS.get_solution(MS), U0);
@@ -330,7 +336,7 @@ void calcul_von_mises(const getfem::mesh_fem &mf_u, const VEC1 &U,
     vm_max = (i == 0 ? VM[0] : std::max(vm_max, VM[i]));
     assert(VM[i] > -1e-6);
   }
-  cout << "Von Mises : min=" << 4*mu*mu*vm_min << ", max=" << 4*mu*mu*vm_max << "\n";
+  // cout << "Von Mises : min=" << 4*mu*mu*vm_min << ", max=" << 4*mu*mu*vm_max << "\n";
   gmm::scale(VM, 4*mu*mu);
 }
 
@@ -352,6 +358,7 @@ void friction_problem::solve(void) {
     if (mf_u.point_of_basic_dof(i)[N-1]
 	< mf_u.point_of_basic_dof(ref_dof)[N-1])
       ref_dof = i;
+  cout << "ref_dof selected : " << mf_u.point_of_basic_dof(ref_dof) << endl;
 
   // Linearized elasticity brick.
   getfem::mdbrick_isotropic_linearized_elasticity<>
@@ -396,7 +403,8 @@ void friction_problem::solve(void) {
 
   getfem::mdbrick_Coulomb_friction<>
     FRICTION(DIRICHLET, BN, gap,
-	     friction_coef * ((scheme == 3) ? (1./theta) : 1.), BT);
+	     //	     friction_coef * ((scheme == 3) ? (1./theta) : 1.), BT);
+	     friction_coef, BT);
   FRICTION.set_r(r);
 
   // Dynamic brick.
@@ -453,6 +461,7 @@ void friction_problem::solve(void) {
   scalar_type alpha_(0), J_friction0(0), J_friction1(0);
   plain_vector one(mf_u.nb_dof());
   std::fill(one.begin(), one.end(), 1.0);
+  plain_vector UU1(N);
 
   sparse_matrix BBT(gmm::mat_nrows(BN), gmm::mat_nrows(BN));
   if (scheme == 4 || scheme == 7) gmm::mult(BN, gmm::transposed(BN), BBT);
@@ -475,7 +484,7 @@ void friction_problem::solve(void) {
   }
  
   gmm::clear(MA0);
-  gmm::iteration iter(residual, 0, 40000);
+  gmm::iteration iter(residual, 0, 10000);
   if ((scheme == 0 || scheme == 1) && !init_stationary) {
     plain_vector FA(mf_u.nb_dof());
     gmm::mult(ELAS.get_K(), gmm::scaled(U0, -1.0),
@@ -517,7 +526,7 @@ void friction_problem::solve(void) {
 		       - gmm::vect_sp(VOL_F.get_F(), U0));
   cout << "t=0, initial energy: " << Einit << endl;
   
-  while (t <= T) {
+  while (t <= T-dt+1e-12) {
 
     switch (scheme) { // complementary left hand side and velocity complement
     case 0 :
@@ -549,12 +558,22 @@ void friction_problem::solve(void) {
       break;
     case 3 : // for the friction, it should be better to take the average 
       // for the contact forces to define the friction threshold
-      a = 2./(dt*dt); b = 1.;  beta_ = 1./dt; alpha_ = 1.;
-      gmm::add(gmm::scaled(U0, a), gmm::scaled(V0, 2./dt), U1);
+//       a = 2./(dt*dt); b = 1.;  beta_ = 1./dt; alpha_ = 1.;
+//       gmm::add(gmm::scaled(U0, a), gmm::scaled(V0, 2./dt), U1);
+//       gmm::mult(DYNAMIC.get_M(), U1, DF);
+//       gmm::mult_add(gmm::transposed(BT), gmm::scaled(LN0, (1.-theta)), DF);
+//       gmm::copy(gmm::scaled(U0, -1.), WT);
+//       gmm::clear(WN);
+
+      a = 4./(dt*dt); b = 1.;  beta_ = 2./dt; alpha_ = 1.;
+      gmm::add(gmm::scaled(U0, a), gmm::scaled(V0, 4./dt), U1);
       gmm::mult(DYNAMIC.get_M(), U1, DF);
-      gmm::mult_add(gmm::transposed(BT), gmm::scaled(LN0, (1.-theta)), DF);
       gmm::copy(gmm::scaled(U0, -1.), WT);
       gmm::clear(WN);
+
+
+
+
       break;
     case 4 : // Paoli-Schatzman scheme DF --> F^n
       gmm::mult(ELAS.get_K(), gmm::scaled(U1, -1.),
@@ -620,14 +639,19 @@ void friction_problem::solve(void) {
       	  if (UN[i] < -1E-13) FRICTION.get_gap()[i] = 1.E13;
       }
   
-      gmm::default_newton_line_search ls;
+      getfem::default_newton_line_search ls;
+      // getfem::simplest_newton_line_search ls(100000, 1.4, 0.001, 0.8);
       getfem::standard_solve(MS, PERIODIC, iter,
 			     getfem::default_linear_solver(PERIODIC), ls);
     
-      if (scheme  == 5 || scheme == 6)
+      if (scheme  == 5 || scheme == 6) {
 	gmm::copy(ELAS.get_solution(MS), U2);
-      else
+	gmm::copy(gmm::sub_vector(U2, gmm::sub_interval(ref_dof, N)), UU1);
+      } else {
 	gmm::copy(ELAS.get_solution(MS), U1);
+	gmm::copy(gmm::sub_vector(U1, gmm::sub_interval(ref_dof, N)), UU1);
+      }
+
       gmm::copy(FRICTION.get_LN(MS), LN1);
       gmm::copy(FRICTION.get_LT(MS), LT1);
     }
@@ -676,11 +700,18 @@ void friction_problem::solve(void) {
       }
       break;
     case 3 :
-      gmm::scale(LN1, 1./theta);
+//       gmm::scale(LN1, 1./theta);
+//       gmm::add(gmm::scaled(U1, 2./dt), gmm::scaled(U0, -2./dt), V1);
+//       gmm::add(gmm::scaled(V0, -1), V1);
+//       J_friction1 = J_friction0 + dt * gmm::vect_sp(BT, V1, LT1);
+
+      // gmm::scale(LN1, 0.5);
+      // gmm::scale(LT1, 0.5);
       gmm::add(gmm::scaled(U1, 2./dt), gmm::scaled(U0, -2./dt), V1);
-      J_friction1 = J_friction0 + dt * 0.5 * gmm::vect_sp(BT,V1, LT1);
       gmm::add(gmm::scaled(V0, -1), V1);
+      J_friction1 = J_friction0 + dt * gmm::vect_sp(BT,V1, LT1);
       break;
+
     case 4 :
       if (t == 0)
 	gmm::add(gmm::scaled(V0, dt), U0, U1); // + terme d'ordre superieur ...
@@ -734,7 +765,7 @@ void friction_problem::solve(void) {
 
     if (scheme != 5 && scheme != 6){
       kenetic_nrj = 0.5 * gmm::vect_sp(DYNAMIC.get_M(), V1, V1);
-      elastic_nrj = 0.5*gmm::vect_sp(ELAS.get_K(), U1, U1);
+      elastic_nrj = 0.5 * gmm::vect_sp(ELAS.get_K(), U1, U1);
       potential_nrj = - gmm::vect_sp(VOL_F.get_F(), U1);
       J1 = kenetic_nrj + elastic_nrj + potential_nrj;
     }
@@ -762,7 +793,7 @@ void friction_problem::solve(void) {
 	}
       }
 
-      cout << "r = " << FRICTION.get_r() << endl;
+      // cout << "r = " << FRICTION.get_r() << endl;
       cout << "t = " << t << " energy : ";
       if (scheme == 5 || scheme == 6) cout << Jdemi;
       else if (scheme == 2) cout << J1 << " energy at midpoint : " << Jdemi;
@@ -778,31 +809,29 @@ void friction_problem::solve(void) {
       gmm::copy(LN1, LN0); gmm::copy(LT1, LT0); J_friction0 = J_friction1;
       if (scheme == 4 || scheme == 5 || scheme == 6) gmm::copy(U2, U1);
       if (dxexport && t >= t_export-dt/20.0) {
-	plain_vector UU1(N), VV1(N);
-	plain_vector LLN1(gmm::vect_size(U0));
+	plain_vector LLN1(gmm::vect_size(U0)), LLT1(gmm::vect_size(U0));
 	gmm::mult(gmm::transposed(BN), LN1, LLN1);
-
-
+	gmm::mult(gmm::transposed(BT), LT1, LLT1);
 
 	//	scalar_type h = mesh.minimal_convex_radius_estimate();
 	// Pas bon, il faut multiplier par l'inverse de la matrice de masse
 	// sur le bord.
-	scalar_type rr = MM(ref_dof+N-1, ref_dof+N-1);
+//	scalar_type rr = MM(ref_dof+N-1, ref_dof+N-1);
 // 	cout << "\n\n" ;
 // 	cout << "rr = " << rr << "\n\n"; 
 // 	cout << "LLN1[ref_dof+N-1] = " << LLN1[ref_dof+N-1] <<"\n\n";
-	LLN1[ref_dof+N-1] = rho*LLN1[ref_dof+N-1]/rr; 
+	LLN1[ref_dof+N-1] = LLN1[ref_dof+N-1]; 
+	LLT1[ref_dof] = LLT1[ref_dof]; 
 // 	cout << "rho/rr " << rho/rr <<"\n\n";
 // 	cout << "LLN1 = " <<  LLN1[ref_dof+N-1] <<  "\n\n";
 
-	gmm::copy(gmm::sub_vector(U1, gmm::sub_interval(ref_dof, N)), UU1);
-	gmm::copy(gmm::sub_vector(V1, gmm::sub_interval(ref_dof, N)), VV1);
 
  	fileout << t << "\t";
 	fileout << J1   << "\t";	       // Total energy
 	fileout << J_friction1  << "\t";       // Energy dissipated by friction
  	fileout << -LLN1[ref_dof+N-1] << "\t"; // contact pressure at A
- 	fileout << UU1[N-1] << "\n";           // normal displacement at A
+ 	fileout << -LLT1[ref_dof] << "\t";     // contact friction at A
+ 	fileout << UU1[N-1] << endl;           // normal displacement at A
 
 	exp->write_point_data(mf_u, U0);
 	exp->serie_add_object("deformationsteps");
@@ -815,6 +844,39 @@ void friction_problem::solve(void) {
       }
     }
   }
+
+
+  // save the last time step for convergence test.
+  mf_u.write_to_file(datafilename + ".mfu", true);
+  gmm::vecsave(datafilename + ".U", U0);
+  
+  if (compare) {
+    getfem::mesh m_ref;
+    m_ref.read_from_file(refrootname + ".mfu");
+    getfem::mesh_fem mf_ref(m_ref);
+    mf_ref.read_from_file(refrootname + ".mfu");
+    plain_vector Uref(mf_ref.nb_dof());
+    gmm::vecload(refrootname + ".U", Uref);
+    
+    plain_vector U(mf_ref.nb_dof());
+    
+    getfem::interpolation(mf_u, mf_ref, U0, U, true);
+    getfem::mesh_im mim_ref(m_ref);
+    getfem::pintegration_method ppi = 
+      getfem::int_method_descriptor(INTEGRATION);
+    mim_ref.set_integration_method(m_ref.convex_index(), ppi);
+    
+    cout << "To ref L2 ERROR:"
+	 << getfem::asm_L2_dist(mim_ref, mf_ref, U, mf_ref, Uref) << endl;
+    
+    cout << "To ref H1 ERROR:"
+	 << getfem::asm_H1_dist(mim_ref, mf_ref, U, mf_ref, Uref) << endl;
+    
+    cout << "H1 Norm:"
+	 << getfem::asm_H1_norm(mim_ref, mf_ref, Uref) << endl;
+  }
+  
+  
 }
   
 /**************************************************************************/
@@ -823,12 +885,6 @@ void friction_problem::solve(void) {
 
 int main(int argc, char *argv[]) {
 
-//   /* In order to test locale. */
-//   setlocale(LC_NUMERIC,"");
-//   std::locale lc("");
-//   cin.imbue(lc);
-//   cout.imbue(lc);
-
 
   GMM_SET_EXCEPTION_DEBUG; // Exceptions make a memory fault, to debug.
   FE_ENABLE_EXCEPT;        // Enable floating point exception for Nan.
